# File:         scanner.py
# Author:       Brian Allen Vanderburg II
# Purpose:      The scanner class scans a source directory
# License:      Refer to the file license.txt

import sys
import os
import re

from lxml import etree

class _Match(object):
    def __init__(self, xml):
        self.ending = xml.get('ending', '.xml')
        self.action = xml.get('action', '')
        self.builder = xml.get('builder', '')

    @staticmethod
    def load(xml):
        return _Match(xml)

class Scanner(object):
    def __init__(self, xml):
        """ Parse and load the scanner """
        self.source = xml.get('source')
        self.target = xml.get('target')

        if self.source:
            self.source = self.source.replace('/', os.sep)

        if self.target:
            self.target = self.target.replace('/', os.sep)

        self.profiles = []
        for i in xml.findall('profile'):
            profile = i.get('name')
            if profile:
                self.profiles.append(profile)

        self.includes = []
        for i in xml.findall('include'):
            pattern = i.get('pattern')
            if pattern:
                self.includes.append(pattern)

        self.excludes = []
        for i in xml.findall('exclude'):
            pattern = i.get('pattern')
            if pattern:
                self.excludes.append(pattern)

        self.matches = []
        for i in xml.findall('match'):
            self.matches.append(_Match.load(i))

        self.params = {}
        for i in xml.findall('param'):
            name = i.get('name')
            value = i.get('value')
            if name and value:
                self.params[name] = value

    @staticmethod
    def load(xml):
        return Scanner(xml)

    def execute(self, profile, params):
        from .config import Config

        if len(self.profiles) > 0 and not profile in self.profiles:
            return

        source = os.path.normpath(os.path.join(Config.confdir, self.source))
        target = os.path.normpath(os.path.join(Config.confdir, self.target))

        for (dir, dirs, files) in os.walk(source):
            for f in files:
                relpath = os.path.relpath(os.path.join(dir, f), source)
                compare = relpath.replace(os.sep, '/')

                # Includes
                found = any([re.search(i, compare) for i in self.includes])
                if not found and len(self.includes) > 0:
                    continue

                # Excludes
                if any([re.search(e, compare) for e in self.excludes]):
                    continue

                # Execute the matches
                for match in self.matches:
                    if relpath.endswith(match.ending) or len(match.ending) == 0:
                        # Execute the builder
                        builder = Config.builders.get(match.builder, None)
                        if builder:
                            bparams = self.params.copy()
                            bparams.update(params)
                            builder.execute(profile, bparams, source, target, relpath, match.ending)

                        # Execute the action if any
                        if match.action == 'link':
                            sourcefile = os.path.join(source, relpath)
                            targetfile = os.path.join(target, relpath)
                            targetdir = os.path.dirname(targetfile)

                            if not os.path.isdir(targetdir):
                                os.makedirs(targetdir)
                            elif os.path.exists(targetfile):
                                os.unlink(targetfile)

                            link = os.path.relpath(sourcefile, targetdir)
                            os.symlink(link, targetfile)



